
% -------------------------------------------------------------------------
% Shape Constrained Splines toolbox - Demo_Titanium.m
% -------------------------------------------------------------------------
% Description 
%
% This script demonstrates the use of the SCS and SPIKE_O toolboxes for
% shape constrained spline function fitting. 
%
% -------------------------------------------------------------------------
% Last modification: Kris Villez, 2015-05-11
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2012-2016 Kris Villez
%
% This file is part of a case study in Matlab/Octave. 
% 
% This program is free software: you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your 
% option) any later version.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

clc 
clear all
close all

currentpath =cd ;
cd ..
cd ..
addpath(genpath([cd '\Adopted']))
addpath(genpath([cd '\SCS_Tools']))
cd(currentpath)

% =========================================================================

disp('0. Load and plot data:')
load data_titanium

plottime = x(1):.01:x(end);

figure, plot(x,y,'ko')

disp(' paused')
pause

% =========================================================================

disp('1. Create SCS object and fit unconstrained spline:')
tk      =       x(1:1:end)          ;
tx      =   [   tk(1):1:tk(end) ]   ;
tk      =   [   tk([1:1:end-1]) tk(end)    ]   ;


lambda      =   [   0 0  10^(-7) 0  ]   ;
order       =   4 ; 
model        =   CreateSCS(y(:),x(:),tk(:),order,lambda)       ;

model1        =   FitSCS(model)        ;

figure, PlotSCS(model1,plottime,[0 1]);

disp(' 	Notice the fluctuations in the 1st derivative of the fitted function')
disp(' paused')
pause

% =========================================================================

disp('2. Add shape constraint as in Turlach et al. (2005) and fit Shape Constrained Spline accordingly:') 

model2  =       model ;
QR      =   [   595 835 955     ;   % episode start
                835 955 1075    ;   % episode end
                nan(1,3)        ;   % sign for function value
                nan(1,3)        ;   % sign for 1st derivative
                1   nan  1      ;   % sign for 2nd derivative
                nan(1,3)        ;   % sign for 3rd derivative
                zeros(1,3)    ] ;   % 0/1 indicator for discontinuity (currently not supported, all zero)
QR      =   QR' ;            

model2  =   ApplyQR(model2,QR)	;

disp('	Applied QR: ')            
disp(['      Episode start: ' num2str(QR(:,1)','%5.0f') ])
disp(['      Episode end:   ' num2str(QR(:,2)','%5.0f') ])            
disp(['      Primitive:     ' ' P    Q    P'] )            

model2        =   FitSCS(model2)    ;
figure, PlotSCS(model2,plottime,[0 1 ]);

disp(' 	Notice how the fluctuations in the 1st derivative of the fitted function have disappeared')
disp(' paused')
pause

% =========================================================================

disp('3. Compute bounds:') 

QR      =   [   595 886 955     ;   % episode start
                835 910 1075    ;   % episode end
                nan(1,3)        ;   % sign for function value
                1 nan -1        ;   % sign for 1st derivative
                1   -1  1       ;   % sign for 2nd derivative
                nan(1,3)        ;   % sign for 3rd derivative
                zeros(1,3)    ] ;   % 0/1 indicator for discontinuity (currently not supported, all zero)
QR      =   QR'     ;

model3  =   model               ;
model3  =   ApplyQR(model3,QR)	;

graph   =   1   ;
verify  =   0   ;
IntX    =   model.tk([ 1 1 ; end  end]); 
DistX   =   []  ;

disp('	Applied intervals for transitions: ')            
disp([ '      Transition 1: ' num2str(IntX(:,1)','%5.0f')])    
disp([ '      Transition 2: ' num2str(IntX(:,2)','%5.0f')])

[FL,FU,SolX,model3] = BoundSCS(model3,IntX,DistX,graph,verify);

disp('	Computed solution for upper bound: ')            
disp([ '      Transition 1: ' num2str(SolX(1)','%5.2f')])    
disp([ '      Transition 2: ' num2str(SolX(2)','%5.2f')])
disp('')
disp('	Applied QR for upper bound: ')            
disp(['      Episode start: ' num2str([model3.QR(1,1) ; SolX]','%5.0f') ])
disp(['      Episode end:   ' num2str([SolX ; model3.QR(end,2)]','%5.0f') ])            
disp(['      Primitive:     ' ' B    N    A'] )        
disp('') 
disp('	Applied QR for lower bound: ')            
disp(['      Episode start: ' num2str([model3.QR(1,1) IntX(2,:)],'%5.0f') ])
disp(['      Episode end:   ' num2str([IntX(1,:) model3.QR(end,2) ],'%5.0f') ])            
disp(['      Primitive:     ' ' B    N    A'] )        

disp('	Computed bounds: ')            
disp([ '      Lower bound: ' num2str(FL,'%5.6f')])    
disp([ '      Upper bound: ' num2str(FU,'%5.6f')])


disp(' paused')
pause

% =========================================================================
% Branch-and-Bound
disp('4. Branch-and-bound search:') 

model4      =   model    ;
model4  =   ApplyQR(model4,QR)	;

nk          =   length(model4.tk)               ;
nt          =   size(model4.QR,1)-1             ;
IntX        =   model4.tk([1 nk]'*ones(1,nt))    ;
DistX       =   []                              ;

disp('	Applied root set for transitions: ')            
disp([ '      Transition 1: ' num2str(IntX(:,1)','%5.0f')])    
disp([ '      Transition 2: ' num2str(IntX(:,2)','%5.0f')])

options         =   OptimsetBB(model4)  ;

disp('')
disp('	Colour code: ')    
disp('      Blue    = fathomed node')    
disp('      Pink    = verified solution (upper bound = lower bound)')    
disp('      Black   = selected solution')    
disp('  Please take note of above colour codes to understand subsequent visualization of the branch-and-bound algorithm')    
disp(' paused')
pause

options.resolution = 1;
[Tree,model4]   =   BB(model4,options,IntX,DistX)    ;

[QR4,trans] = SCS2QR(model4);
 
disp('')
disp('	Obtained optimal solution: ')            
disp([ '      Transition 1: ' num2str(trans(1)','%5.3f')])    
disp([ '      Transition 2: ' num2str(trans(2)','%5.3f')])

disp(' paused')

Xlim    =   [model.tk([1 end])]*model.t_scale+model.t_ref ;
figure(999)
subplot(2,1,1)
    %set(gca,'Xtick',[plottime(1):60:plottime(end)])
    %set(gca,'Ytick',[plottime(1):60:plottime(end)])
    if exist('xticklabel_rotate')
        % if interested,get this function at:
        %   http://www.mathworks.com/matlabcentral/fileexchange/3486-xticklabelrotate
        hText = xticklabel_rotate;
    end
    PlotSCS(model4,plottime);    

    set(gca,'Xlim',Xlim)
    ylabel('y');

subplot(2,1,2)
    PlotQRT(QR4,3);
    
    set(gca,'Xlim',Xlim)
    xlabel('Temperature [^oC]');
    ylabel('QR');

disp('	Plotted optimal solution with qualitative representation')            

return
